import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from sklearn.linear_model import LinearRegression as LRbuiltin
from sklearn.model_selection import train_test_split

class LinearnaRegresija:

    def __init__(self, podaci, x, y):
        self.podaci = podaci
        self.x = self.podaci[x]
        self.y = self.podaci[y]

    def prikaz_grafa(self, boja = 'g', naslov = 'Inicijalni prikaz'):
        plt.scatter(x = self.x, y = self.y, color = boja)
        plt.title(naslov)
        plt.show()
        return

    def podjela_seta(self, omjer_testa : float = 0.2, slučajno_stanje : int = 50):
        self.x_train, self.x_test, self.y_train, self.y_test = train_test_split(self.x, self.y, test_size = omjer_testa, random_state = slučajno_stanje)
        return

    def prikaz_procesa(self, tr_naslov = 'Uzorak treniranja', te_naslov = 'Uzorak testiranja', tr_boja = 'r', te_boja = 'g'):
        plt.scatter(self.x_train, self.y_train, label = tr_naslov, color = tr_boja, alpha=.3)
        plt.scatter(self.x_test, self.y_test, label = te_naslov, color = te_boja, alpha=.3)
        plt.legend()
        plt.title('Proces treniranja i testiranja modela', fontweight = 'bold')
        plt.show()

    def prikaz_linearne_regresije(self):
        self.LR = LRbuiltin()
        self.LR.fit(self.x_train.values.reshape(-1,1), self.y_train.values)
        predviđanje = self.LR.predict(self.x_test.values.reshape(-1,1))
        plt.plot(self.x_test, predviđanje, label = 'Linearna regresija',)
        plt.scatter(self.x_test, self.y_test, label = 'Vrijednosti', color = 'darkkhaki', alpha = 0.5)
        plt.legend()
        plt.title('Linearna regresija', fontweight = 'bold')
        plt.show()

    def procjena_varijable(self, vrijednost, prikaz = True):
        self.vrijednost = vrijednost
        procjena = self.LR.predict(np.array([[vrijednost]]))[0]
        if prikaz == True:
            return print(f'Odgovarajuća vrijednost za {vrijednost} je {procjena}.')
        else:
            return procjena

    def preciznost_modela(self, prikaz = True):
        preciznost = self.LR.score(self.x_test.values.reshape(-1,1), self.y_test.values)
        if prikaz == True:
            return print(f'Uspješnost modela iznosi {preciznost}.')
        else:
            return preciznost
    
    def formula(self, prikaz = True):
        reformat_coef = float(self.LR.coef_)
        if prikaz == True:
            return print(f'Formula modela je y = {self.vrijednost} * {reformat_coef} + {self.LR.intercept_}')
        else:
            return f'Formula modela je y = {self.vrijednost} * {reformat_coef} + {self.LR.intercept_}'



from sklearn.cluster import KMeans
from sklearn.preprocessing import MinMaxScaler
from matplotlib import pyplot as plt
import pandas as pd
import random

class KSrednjeVrijednosti:

    def __init__(self, datoteka, broj_klastera, koordinate):
        self.datoteka = pd.DataFrame(datoteka)
        self.originalna_datoteka = pd.DataFrame(datoteka)
        self.koordinate = koordinate
        self.broj_klastera = broj_klastera # Maksimalno 20.

    def podesi_omjer(self,):
        omjer = MinMaxScaler()
        for col in self.koordinate:
            omjer.fit(self.datoteka[[col]])
            self.datoteka[col] = omjer.transform(self.datoteka[[col]])
        
        return self.datoteka

    def kreiraj_klastere(self,):
        self.ksv = KMeans(n_clusters = self.broj_klastera)
        self.dodjela_klastera = self.ksv.fit_predict(self.datoteka[self.koordinate]) # ispitati
        self.datoteka['Klaster'] = self.dodjela_klastera
        
    def prikaz_datoteke(self):
        return self.datoteka

    def prikaz_centroida(self):
        return self.ksv.cluster_centers_

    def prikaz_grafa(self,):
        x = self.koordinate[0]
        y = self.koordinate[1]
        boje = ['r', 'g', 'b', 'c', 'y', 'k', 'brown', 'wheat', 'maroon', 'grey', 'steelblue', 'peru', 'indigo', 'pink', 'darkkhaki', 'deeppink', 'chocolate', 'rosybrown', 'gold', 'tan']
        brojac = 0
        for i in range(self.broj_klastera):
            klaster = self.datoteka[self.datoteka['Klaster'] == brojac]
            tmp_boja = random.choice(boje)
            plt.scatter(klaster[x], klaster[y], color = tmp_boja)
            boje.remove(tmp_boja)
            brojac += 1
        plt.scatter(self.ksv.cluster_centers_[:, 0], self.ksv.cluster_centers_[:, 1], color = 'm', marker = '*', label = 'centroid')
        plt.title(f'{self.broj_klastera} klastera', fontweight = 'bold')
        plt.legend()
        plt.show()

    def prikaz_promjene(self, raspon = range(1,10)):
        self.inercija = []
        self.raspon = raspon
        for k in self.raspon:
            self.tmp_ksv = KMeans(n_clusters = k)
            self.tmp_ksv.fit(self.originalna_datoteka[self.koordinate])
            self.inercija.append(self.tmp_ksv.inertia_)

        plt.title('Prikaz stupnja optimizacije po vrijednosti', fontweight = 'bold')
        plt.xlabel('Broj K-vrijednosti')
        plt.ylabel('Razina odstupanja')
        plt.plot(self.raspon, self.inercija)
        plt.show()
